@@ -23,102 +23,17 @@ from openstack.compute.v2 import server_interface as _server_interface
 from openstack.compute.v2 import server_ip
 from openstack.compute.v2 import service as _service
 from openstack.compute.v2 import volume_attachment as _volume_attachment
-from openstack import proxy2
-from openstack import resource2
+from openstack import proxy
+from openstack import resource
 
 
-class Proxy(proxy2.BaseProxy):
+class Proxy(proxy.Proxy):
 
-    def find_extension(self, name_or_id, ignore_missing=True):
-        """Find a single extension
-
-        :param name_or_id: The name or ID of an extension.
-        :param bool ignore_missing: When set to ``False``
-                    :class:`~openstack.exceptions.ResourceNotFound` will be
-                    raised when the resource does not exist.
-                    When set to ``True``, None will be returned when
-                    attempting to find a nonexistent resource.
-        :returns: One :class:`~openstack.compute.v2.extension.Extension` or
-                  None
-        """
-        return self._find(extension.Extension, name_or_id,
-                          ignore_missing=ignore_missing)
-
-    def extensions(self):
-        """Retrieve a generator of extensions
-
-        :returns: A generator of extension instances.
-        :rtype: :class:`~openstack.compute.v2.extension.Extension`
-        """
-        return self._list(extension.Extension, paginated=False)
-
-    def find_flavor(self, name_or_id, ignore_missing=True):
-        """Find a single flavor
-
-        :param name_or_id: The name or ID of a flavor.
-        :param bool ignore_missing: When set to ``False``
-                    :class:`~openstack.exceptions.ResourceNotFound` will be
-                    raised when the resource does not exist.
-                    When set to ``True``, None will be returned when
-                    attempting to find a nonexistent resource.
-        :returns: One :class:`~openstack.compute.v2.flavor.Flavor` or None
-        """
-        return self._find(_flavor.Flavor, name_or_id,
-                          ignore_missing=ignore_missing)
-
-    def create_flavor(self, **attrs):
-        """Create a new flavor from attributes
-
-        :param dict attrs: Keyword arguments which will be used to create
-                           a :class:`~openstack.compute.v2.flavor.Flavor`,
-                           comprised of the properties on the Flavor class.
-
-        :returns: The results of flavor creation
-        :rtype: :class:`~openstack.compute.v2.flavor.Flavor`
-        """
-        return self._create(_flavor.Flavor, **attrs)
-
-    def delete_flavor(self, flavor, ignore_missing=True):
-        """Delete a flavor
-
-        :param flavor: The value can be either the ID of a flavor or a
-                       :class:`~openstack.compute.v2.flavor.Flavor` instance.
-        :param bool ignore_missing: When set to ``False``
-                    :class:`~openstack.exceptions.ResourceNotFound` will be
-                    raised when the flavor does not exist.
-                    When set to ``True``, no exception will be set when
-                    attempting to delete a nonexistent flavor.
-
-        :returns: ``None``
-        """
-        self._delete(_flavor.Flavor, flavor, ignore_missing=ignore_missing)
-
-    def get_flavor(self, flavor):
-        """Get a single flavor
-
-        :param flavor: The value can be the ID of a flavor or a
-                       :class:`~openstack.compute.v2.flavor.Flavor` instance.
-
-        :returns: One :class:`~openstack.compute.v2.flavor.Flavor`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
-                 when no resource can be found.
-        """
-        return self._get(_flavor.Flavor, flavor)
-
-    def flavors(self, details=True, **query):
-        """Return a generator of flavors
-
-        :param bool details: When ``True``, returns
-            :class:`~openstack.compute.v2.flavor.FlavorDetail` objects,
-            otherwise :class:`~openstack.compute.v2.flavor.Flavor`.
-            *Default: ``True``*
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
-                                 the flavors being returned.
-
-        :returns: A generator of flavor objects
-        """
-        flv = _flavor.FlavorDetail if details else _flavor.Flavor
-        return self._list(flv, paginated=True, **query)
+    Extension = extension.Extension
+    Flavor = _flavor.Flavor
+    FlavorDetail = _flavor.FlavorDetail
+    Server = _server.Server
+    ServerDetail = _server.ServerDetail
 
     def delete_image(self, image, ignore_missing=True):
         """Delete an image
@@ -198,7 +113,7 @@ class Proxy(proxy2.BaseProxy):
         :rtype: :class:`~openstack.compute.v2.image.Image`
         """
         res = self._get_base_resource(image, _image.Image)
-        metadata = res.get_metadata(self._session)
+        metadata = res.get_metadata(self)
         result = _image.Image.existing(id=res.id, metadata=metadata)
         return result
 
@@ -219,7 +134,7 @@ class Proxy(proxy2.BaseProxy):
         :rtype: :class:`~openstack.compute.v2.image.Image`
         """
         res = self._get_base_resource(image, _image.Image)
-        metadata = res.set_metadata(self._session, **metadata)
+        metadata = res.set_metadata(self, **metadata)
         result = _image.Image.existing(id=res.id, metadata=metadata)
         return result
 
@@ -237,7 +152,7 @@ class Proxy(proxy2.BaseProxy):
         :rtype: ``None``
         """
         res = self._get_base_resource(image, _image.Image)
-        return res.delete_metadata(self._session, keys)
+        return res.delete_metadata(self, keys)
 
     def create_keypair(self, **attrs):
         """Create a new keypair from attributes
@@ -312,18 +227,6 @@ class Proxy(proxy2.BaseProxy):
         """
         return self._get(limits.Limits)
 
-    def create_server(self, **attrs):
-        """Create a new server from attributes
-
-        :param dict attrs: Keyword arguments which will be used to create
-                           a :class:`~openstack.compute.v2.server.Server`,
-                           comprised of the properties on the Server class.
-
-        :returns: The results of server creation
-        :rtype: :class:`~openstack.compute.v2.server.Server`
-        """
-        return self._create(_server.Server, **attrs)
-
     def delete_server(self, server, ignore_missing=True, force=False):
         """Delete a server
 
@@ -341,35 +244,10 @@ class Proxy(proxy2.BaseProxy):
         """
         if force:
             server = self._get_resource(_server.Server, server)
-            server.force_delete(self._session)
+            server.force_delete(self)
         else:
-            self._delete(_server.Server, server, ignore_missing=ignore_missing)
-
-    def find_server(self, name_or_id, ignore_missing=True):
-        """Find a single server
-
-        :param name_or_id: The name or ID of a server.
-        :param bool ignore_missing: When set to ``False``
-                    :class:`~openstack.exceptions.ResourceNotFound` will be
-                    raised when the resource does not exist.
-                    When set to ``True``, None will be returned when
-                    attempting to find a nonexistent resource.
-        :returns: One :class:`~openstack.compute.v2.server.Server` or None
-        """
-        return self._find(_server.Server, name_or_id,
-                          ignore_missing=ignore_missing)
-
-    def get_server(self, server):
-        """Get a single server
-
-        :param server: The value can be the ID of a server or a
-                       :class:`~openstack.compute.v2.server.Server` instance.
-
-        :returns: One :class:`~openstack.compute.v2.server.Server`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
-                 when no resource can be found.
-        """
-        return self._get(_server.Server, server)
+            self._generated_delete_server(
+                server, ignore_missing=ignore_missing)
 
     def servers(self, details=True, **query):
         """Retrieve a generator of servers
@@ -408,8 +286,7 @@ class Proxy(proxy2.BaseProxy):
 
         :returns: A generator of server instances.
         """
-        srv = _server.ServerDetail if details else _server.Server
-        return self._list(srv, paginated=True, **query)
+        return self._generated_servers(details=details, **query)
 
     def update_server(self, server, **attrs):
         """Update a server
@@ -434,7 +311,18 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.change_password(self._session, new_password)
+        server.change_password(self, new_password)
+
+    def get_server_password(self, server):
+        """Get the administrator password
+
+        :param server: Either the ID of a server or a
+                       :class:`~openstack.compute.v2.server.Server` instance.
+
+        :returns: encrypted password.
+        """
+        server = self._get_resource(_server.Server, server)
+        return server.get_password(self._session)
 
     def reset_server_state(self, server, state):
         """Reset the state of server
@@ -447,7 +335,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         res = self._get_base_resource(server, _server.Server)
-        res.reset_state(self._session, state)
+        res.reset_state(self, state)
 
     def reboot_server(self, server, reboot_type):
         """Reboot a server
@@ -460,7 +348,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.reboot(self._session, reboot_type)
+        server.reboot(self, reboot_type)
 
     def rebuild_server(self, server, name, admin_password, **attrs):
         """Rebuild a server
@@ -488,7 +376,7 @@ class Proxy(proxy2.BaseProxy):
                   instance.
         """
         server = self._get_resource(_server.Server, server)
-        return server.rebuild(self._session, name, admin_password, **attrs)
+        return server.rebuild(self, name, admin_password, **attrs)
 
     def resize_server(self, server, flavor):
         """Resize a server
@@ -501,8 +389,8 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        flavor_id = resource2.Resource._get_id(flavor)
-        server.resize(self._session, flavor_id)
+        flavor_id = resource.Resource._get_id(flavor)
+        server.resize(self, flavor_id)
 
     def confirm_server_resize(self, server):
         """Confirm a server resize
@@ -513,7 +401,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.confirm_resize(self._session)
+        server.confirm_resize(self)
 
     def revert_server_resize(self, server):
         """Revert a server resize
@@ -524,7 +412,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.revert_resize(self._session)
+        server.revert_resize(self)
 
     def create_server_image(self, server, name, metadata=None):
         """Create an image from a server
@@ -537,7 +425,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.create_image(self._session, name, metadata)
+        server.create_image(self, name, metadata)
 
     def add_security_group_to_server(self, server, security_group):
         """Add a security group to a server
@@ -551,8 +439,8 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        security_group_id = resource2.Resource._get_id(security_group)
-        server.add_security_group(self._session, security_group_id)
+        security_group_id = resource.Resource._get_id(security_group)
+        server.add_security_group(self, security_group_id)
 
     def remove_security_group_from_server(self, server, security_group):
         """Remove a security group from a server
@@ -566,8 +454,8 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        security_group_id = resource2.Resource._get_id(security_group)
-        server.remove_security_group(self._session, security_group_id)
+        security_group_id = resource.Resource._get_id(security_group)
+        server.remove_security_group(self, security_group_id)
 
     def add_fixed_ip_to_server(self, server, network_id):
         """Adds a fixed IP address to a server instance.
@@ -579,7 +467,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.add_fixed_ip(self._session, network_id)
+        server.add_fixed_ip(self, network_id)
 
     def remove_fixed_ip_from_server(self, server, address):
         """Removes a fixed IP address from a server instance.
@@ -591,7 +479,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.remove_fixed_ip(self._session, address)
+        server.remove_fixed_ip(self, address)
 
     def add_floating_ip_to_server(self, server, address, fixed_address=None):
         """Adds a floating IP address to a server instance.
@@ -605,7 +493,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.add_floating_ip(self._session, address,
+        server.add_floating_ip(self, address,
                                fixed_address=fixed_address)
 
     def remove_floating_ip_from_server(self, server, address):
@@ -618,7 +506,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.remove_floating_ip(self._session, address)
+        server.remove_floating_ip(self, address)
 
     def backup_server(self, server, name, backup_type, rotation):
         """Backup a server
@@ -634,7 +522,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.backup(self._session, name, backup_type, rotation)
+        server.backup(self, name, backup_type, rotation)
 
     def pause_server(self, server):
         """Pauses a server and changes its status to ``PAUSED``.
@@ -644,7 +532,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.pause(self._session)
+        server.pause(self)
 
     def unpause_server(self, server):
         """Unpauses a paused server and changes its status to ``ACTIVE``.
@@ -654,7 +542,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.unpause(self._session)
+        server.unpause(self)
 
     def suspend_server(self, server):
         """Suspends a server and changes its status to ``SUSPENDED``.
@@ -664,7 +552,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.suspend(self._session)
+        server.suspend(self)
 
     def resume_server(self, server):
         """Resumes a suspended server and changes its status to ``ACTIVE``.
@@ -674,7 +562,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.resume(self._session)
+        server.resume(self)
 
     def lock_server(self, server):
         """Locks a server.
@@ -684,7 +572,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.lock(self._session)
+        server.lock(self)
 
     def unlock_server(self, server):
         """Unlocks a locked server.
@@ -694,7 +582,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.unlock(self._session)
+        server.unlock(self)
 
     def rescue_server(self, server, admin_pass=None, image_ref=None):
         """Puts a server in rescue mode and changes it status to ``RESCUE``.
@@ -711,7 +599,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.rescue(self._session, admin_pass=admin_pass,
+        server.rescue(self, admin_pass=admin_pass,
                       image_ref=image_ref)
 
     def unrescue_server(self, server):
@@ -722,7 +610,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.unrescue(self._session)
+        server.unrescue(self)
 
     def evacuate_server(self, server, host=None, admin_pass=None, force=None):
         """Evacuates a server from a failed host to a new host.
@@ -739,7 +627,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.evacuate(self._session, host=host, admin_pass=admin_pass,
+        server.evacuate(self, host=host, admin_pass=admin_pass,
                         force=force)
 
     def start_server(self, server):
@@ -750,7 +638,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.start(self._session)
+        server.start(self)
 
     def stop_server(self, server):
         """Stops a running server and changes its state to ``SHUTOFF``.
@@ -760,7 +648,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.stop(self._session)
+        server.stop(self)
 
     def shelve_server(self, server):
         """Shelves a server.
@@ -775,7 +663,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.shelve(self._session)
+        server.shelve(self)
 
     def unshelve_server(self, server):
         """Unselves or restores a shelved server.
@@ -789,7 +677,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.unshelve(self._session)
+        server.unshelve(self)
 
     def get_server_console_output(self, server, length=None):
         """Return the console output for a server.
@@ -802,12 +690,12 @@ class Proxy(proxy2.BaseProxy):
                     escaped to create a valid JSON string.
         """
         server = self._get_resource(_server.Server, server)
-        return server.get_console_output(self._session, length=length)
+        return server.get_console_output(self, length=length)
 
     def wait_for_server(self, server, status='ACTIVE', failures=['ERROR'],
                         interval=2, wait=120):
-        return resource2.wait_for_status(self._session, server, status,
-                                         failures, interval, wait)
+        return resource.wait_for_status(
+            self, server, status, failures, interval, wait)
 
     def create_server_interface(self, server, **attrs):
         """Create a new server interface from attributes
@@ -822,7 +710,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: The results of server interface creation
         :rtype: :class:`~openstack.compute.v2.server_interface.ServerInterface`
         """
-        server_id = resource2.Resource._get_id(server)
+        server_id = resource.Resource._get_id(server)
         return self._create(_server_interface.ServerInterface,
                             server_id=server_id, **attrs)
 
@@ -848,7 +736,7 @@ class Proxy(proxy2.BaseProxy):
         """
         server_id = self._get_uri_attribute(server_interface, server,
                                             "server_id")
-        server_interface = resource2.Resource._get_id(server_interface)
+        server_interface = resource.Resource._get_id(server_interface)
 
         self._delete(_server_interface.ServerInterface,
                      port_id=server_interface,
@@ -874,7 +762,7 @@ class Proxy(proxy2.BaseProxy):
         """
         server_id = self._get_uri_attribute(server_interface, server,
                                             "server_id")
-        server_interface = resource2.Resource._get_id(server_interface)
+        server_interface = resource.Resource._get_id(server_interface)
 
         return self._get(_server_interface.ServerInterface,
                          server_id=server_id, port_id=server_interface)
@@ -888,7 +776,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: A generator of ServerInterface objects
         :rtype: :class:`~openstack.compute.v2.server_interface.ServerInterface`
         """
-        server_id = resource2.Resource._get_id(server)
+        server_id = resource.Resource._get_id(server)
         return self._list(_server_interface.ServerInterface, paginated=False,
                           server_id=server_id)
 
@@ -903,7 +791,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: A generator of ServerIP objects
         :rtype: :class:`~openstack.compute.v2.server_ip.ServerIP`
         """
-        server_id = resource2.Resource._get_id(server)
+        server_id = resource.Resource._get_id(server)
         return self._list(server_ip.ServerIP, paginated=False,
                           server_id=server_id, network_label=network_label)
 
@@ -938,7 +826,7 @@ class Proxy(proxy2.BaseProxy):
         :rtype: :class:`~openstack.compute.v2.server.Server`
         """
         res = self._get_base_resource(server, _server.Server)
-        metadata = res.get_metadata(self._session)
+        metadata = res.get_metadata(self)
         result = _server.Server.existing(id=res.id, metadata=metadata)
         return result
 
@@ -959,7 +847,7 @@ class Proxy(proxy2.BaseProxy):
         :rtype: :class:`~openstack.compute.v2.server.Server`
         """
         res = self._get_base_resource(server, _server.Server)
-        metadata = res.set_metadata(self._session, **metadata)
+        metadata = res.set_metadata(self, **metadata)
         result = _server.Server.existing(id=res.id, metadata=metadata)
         return result
 
@@ -977,7 +865,7 @@ class Proxy(proxy2.BaseProxy):
         :rtype: ``None``
         """
         res = self._get_base_resource(server, _server.Server)
-        return res.delete_metadata(self._session, keys)
+        return res.delete_metadata(self, keys)
 
     def create_server_group(self, **attrs):
         """Create a new server group from attributes
@@ -1096,7 +984,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         service = self._get_resource(_service.Service, service)
-        service.force_down(self._session, host, binary)
+        service.force_down(self, host, binary)
 
     def disable_service(self, service, host, binary, disabled_reason=None):
         """Disable a service
@@ -1110,7 +998,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         service = self._get_resource(_service.Service, service)
-        service.disable(self._session,
+        service.disable(self,
                         host, binary,
                         disabled_reason)
 
@@ -1126,7 +1014,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         service = self._get_resource(_service.Service, service)
-        service.enable(self._session, host, binary)
+        service.enable(self, host, binary)
 
     def services(self):
         """Return a generator of service
@@ -1150,7 +1038,7 @@ class Proxy(proxy2.BaseProxy):
         :rtype:
             :class:`~openstack.compute.v2.volume_attachment.VolumeAttachment`
         """
-        server_id = resource2.Resource._get_id(server)
+        server_id = resource.Resource._get_id(server)
         return self._create(_volume_attachment.VolumeAttachment,
                             server_id=server_id, **attrs)
 
@@ -1177,7 +1065,7 @@ class Proxy(proxy2.BaseProxy):
         """
         server_id = self._get_uri_attribute(volume_attachment, server,
                                             "server_id")
-        volume_attachment = resource2.Resource._get_id(volume_attachment)
+        volume_attachment = resource.Resource._get_id(volume_attachment)
 
         return self._update(_volume_attachment.VolumeAttachment,
                             attachment_id=volume_attachment,
@@ -1206,7 +1094,7 @@ class Proxy(proxy2.BaseProxy):
         """
         server_id = self._get_uri_attribute(volume_attachment, server,
                                             "server_id")
-        volume_attachment = resource2.Resource._get_id(volume_attachment)
+        volume_attachment = resource.Resource._get_id(volume_attachment)
 
         self._delete(_volume_attachment.VolumeAttachment,
                      attachment_id=volume_attachment,
@@ -1239,7 +1127,7 @@ class Proxy(proxy2.BaseProxy):
         """
         server_id = self._get_uri_attribute(volume_attachment, server,
                                             "server_id")
-        volume_attachment = resource2.Resource._get_id(volume_attachment)
+        volume_attachment = resource.Resource._get_id(volume_attachment)
 
         return self._get(_volume_attachment.VolumeAttachment,
                          server_id=server_id,
@@ -1256,7 +1144,7 @@ class Proxy(proxy2.BaseProxy):
         :rtype:
             :class:`~openstack.compute.v2.volume_attachment.VolumeAttachment`
         """
-        server_id = resource2.Resource._get_id(server)
+        server_id = resource.Resource._get_id(server)
         return self._list(_volume_attachment.VolumeAttachment, paginated=False,
                           server_id=server_id)
 
@@ -1268,7 +1156,7 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.migrate(self._session)
+        server.migrate(self)
 
     def live_migrate_server(self, server, host=None, force=False):
         """Migrate a server from one host to target host
@@ -1281,4 +1169,4 @@ class Proxy(proxy2.BaseProxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        server.live_migrate(self._session, host, force)
+        server.live_migrate(self, host, force)
