@@ -10,7 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import testtools
+from openstack.tests.unit import base
 
 from openstack.compute.v2 import flavor
 
@@ -30,7 +30,7 @@ BASIC_EXAMPLE = {
 }
 
 
-class TestFlavor(testtools.TestCase):
+class TestFlavor(base.TestCase):
 
     def test_basic(self):
         sot = flavor.Flavor()
@@ -40,14 +40,16 @@ class TestFlavor(testtools.TestCase):
         self.assertEqual('compute', sot.service.service_type)
         self.assertTrue(sot.allow_create)
         self.assertTrue(sot.allow_get)
-        self.assertTrue(sot.allow_update)
         self.assertTrue(sot.allow_delete)
         self.assertTrue(sot.allow_list)
+        self.assertFalse(sot.allow_update)
 
         self.assertDictEqual({"sort_key": "sort_key",
                               "sort_dir": "sort_dir",
                               "min_disk": "minDisk",
-                              "min_ram": "minRam"},
+                              "min_ram": "minRam",
+                              "limit": "limit",
+                              "marker": "marker"},
                              sot._query_mapping._mapping)
 
     def test_make_basic(self):
