@@ -43,11 +43,11 @@ class TestProxyBase(base.TestCase):
 
                 self.assertEqual(expected_result, test_method(*method_args,
                                  **method_kwargs))
-                mocked.assert_called_with(self.session,
+                mocked.assert_called_with(test_method.__self__,
                                           *expected_args, **expected_kwargs)
             else:
                 self.assertEqual(expected_result, test_method())
-                mocked.assert_called_with(self.session)
+                mocked.assert_called_with(test_method.__self__)
 
     # NOTE(briancurtin): This is a duplicate version of _verify that is
     # temporarily here while we shift APIs. The difference is that
@@ -79,10 +79,10 @@ class TestProxyBase(base.TestCase):
                 mocked.assert_called_with(*expected_args, **expected_kwargs)
             else:
                 self.assertEqual(expected_result, test_method())
-                mocked.assert_called_with(self.session)
+                mocked.assert_called_with(test_method.__self__)
 
     def verify_create(self, test_method, resource_type,
-                      mock_method="openstack.proxy.BaseProxy._create",
+                      mock_method="openstack.proxy.Proxy._create",
                       expected_result="result", **kwargs):
         the_kwargs = {"x": 1, "y": 2, "z": 3}
         method_kwargs = kwargs.pop("method_kwargs", the_kwargs)
@@ -98,25 +98,30 @@ class TestProxyBase(base.TestCase):
 
     def verify_delete(self, test_method, resource_type, ignore,
                       input_path_args=None, expected_path_args=None,
-                      mock_method="openstack.proxy.BaseProxy._delete"):
+                      method_kwargs=None, expected_args=None,
+                      expected_kwargs=None,
+                      mock_method="openstack.proxy.Proxy._delete"):
         method_args = ["resource_or_id"]
-        method_kwargs = {"ignore_missing": ignore}
+        method_kwargs = method_kwargs or {}
+        method_kwargs["ignore_missing"] = ignore
         if isinstance(input_path_args, dict):
             for key in input_path_args:
                 method_kwargs[key] = input_path_args[key]
         elif isinstance(input_path_args, list):
             method_args = input_path_args
-        expected_kwargs = {"ignore_missing": ignore}
+        expected_kwargs = expected_kwargs or {}
+        expected_kwargs["ignore_missing"] = ignore
         if expected_path_args:
-            expected_kwargs["path_args"] = expected_path_args
+            expected_kwargs.update(expected_path_args)
+        expected_args = expected_args or [resource_type, "resource_or_id"]
         self._verify2(mock_method, test_method,
                       method_args=method_args,
                       method_kwargs=method_kwargs,
-                      expected_args=[resource_type, "resource_or_id"],
+                      expected_args=expected_args,
                       expected_kwargs=expected_kwargs)
 
     def verify_get(self, test_method, resource_type, value=None, args=None,
-                   mock_method="openstack.proxy.BaseProxy._get",
+                   mock_method="openstack.proxy.Proxy._get",
                    ignore_value=False, **kwargs):
         the_value = value
         if value is None:
@@ -136,8 +141,15 @@ class TestProxyBase(base.TestCase):
                       expected_args=expected_args,
                       expected_kwargs=expected_kwargs)
 
+    def verify_get_overrided(self, proxy, resource_type, patch_target):
+        with mock.patch(patch_target, autospec=True) as res:
+            proxy._get_resource = mock.Mock(return_value=res)
+            proxy._get(resource_type)
+            res.get.assert_called_once_with(proxy, requires_id=True,
+                                            error_message=mock.ANY)
+
     def verify_head(self, test_method, resource_type,
-                    mock_method="openstack.proxy.BaseProxy._head",
+                    mock_method="openstack.proxy.Proxy._head",
                     value=None, **kwargs):
         the_value = [value] if value is not None else []
         expected_kwargs = {"path_args": kwargs} if kwargs else {}
@@ -148,7 +160,7 @@ class TestProxyBase(base.TestCase):
                       expected_kwargs=expected_kwargs)
 
     def verify_find(self, test_method, resource_type, value=None,
-                    mock_method="openstack.proxy.BaseProxy._find",
+                    mock_method="openstack.proxy.Proxy._find",
                     path_args=None, **kwargs):
         method_args = value or ["name_or_id"]
         expected_kwargs = {}
@@ -175,13 +187,11 @@ class TestProxyBase(base.TestCase):
                       **kwargs)
 
     def verify_list(self, test_method, resource_type, paginated=False,
-                    mock_method="openstack.proxy.BaseProxy._list",
+                    mock_method="openstack.proxy.Proxy._list",
                     **kwargs):
         expected_kwargs = kwargs.pop("expected_kwargs", {})
         expected_kwargs.update({"paginated": paginated})
-        expected_kwargs['limit'] = 2
         method_kwargs = kwargs.pop("method_kwargs", {})
-        method_kwargs['limit'] = 2
         self._verify2(mock_method, test_method,
                       method_kwargs=method_kwargs,
                       expected_args=[resource_type],
@@ -191,7 +201,7 @@ class TestProxyBase(base.TestCase):
 
     def verify_list_no_kwargs(self, test_method, resource_type,
                               paginated=False,
-                              mock_method="openstack.proxy.BaseProxy._list"):
+                              mock_method="openstack.proxy.Proxy._list"):
         self._verify2(mock_method, test_method,
                       method_kwargs={},
                       expected_args=[resource_type],
@@ -199,7 +209,7 @@ class TestProxyBase(base.TestCase):
                       expected_result=["result"])
 
     def verify_update(self, test_method, resource_type, value=None,
-                      mock_method="openstack.proxy.BaseProxy._update",
+                      mock_method="openstack.proxy.Proxy._update",
                       expected_result="result", path_args=None, **kwargs):
         method_args = value or ["resource_or_id"]
         method_kwargs = {"x": 1, "y": 2, "z": 3}
